/*
This is a part of the IconDesk source code.

Copyright (C) 2003 Erik Christiansson, aka Sci
www.alphafish.com
erik@alphafish.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "FileIcon.h"

FileIcon::FileIcon()
{
	_pidl = NULL;
}

FileIcon::~FileIcon()
{
	if(_pidl) {
		LPMALLOC pMalloc;
		HRESULT hr;
		hr = SHGetMalloc(&pMalloc);
		if(hr == NOERROR) {
			pMalloc->Free(_pidl);
			pMalloc->Release();
		}
	}
}

void FileIcon::SetPIDL(LPITEMIDLIST pidl)
{
	_pidl = pidl;
}

void FileIcon::Execute(LPCSTR szCommand)
{
	if(!stricmp(szCommand, ".none")) {
		return;
	}
	else if(!stricmp(szCommand, ".context")) {
		PopupContextMenu();
	}
	else if(!stricmp(szCommand, ".open")) {
		ShellExecute(NULL, NULL, filename, NULL, NULL, SW_NORMAL);
	}
}

/**
 * PopupContextMenu
 *
 * Originally stolen from popup2 by the LS dev-team
 */
void FileIcon::PopupContextMenu()
{
	LPMALLOC pMalloc;
	LPSHELLFOLDER psfFolder, psfNextFolder;
	LPITEMIDLIST pidlMain, pidlItem, pidlNextItem, *ppidl;
	LPCONTEXTMENU pContextMenu;
	CMINVOKECOMMANDINFO ici;
	UINT nCount, nCmd;
	BOOL bResult;
	HMENU hMenu;

	pidlMain = MakeCopy(_pidl);

	// Get current cursor position & window
	POINT point;
	HWND hwnd;

	GetCursorPos(&point);
	hwnd = WindowFromPoint(point);

	// Get pointers to the shell's IMalloc interface and the desktop's
	// IShellFolder interface.
	bResult = FALSE;

	if(!SUCCEEDED(SHGetMalloc(&pMalloc))) {
		return;
	}

	if(!SUCCEEDED(SHGetDesktopFolder(&psfFolder))) {
		pMalloc->Release();
		return;
	}

	if(nCount = GetItemCount(pidlMain)) {
		// nCount must be > 0
		// Initialize psfFolder with a pointer to the IShellFolder
		// interface of the folder that contains the item whose context
		// menu we're after, and initialize pidlItem with a pointer to
		// the item's item ID. If nCount > 1, this requires us to walk
		// the list of item IDs stored in pidlMain and bind to each
		// subfolder referenced in the list.
		pidlItem = pidlMain;

		while(--nCount) {
			// Create a 1-item item ID list for the next item in pidlMain.
			pidlNextItem = DuplicateItem(pMalloc, pidlItem);
			if(pidlNextItem == NULL) {
				pMalloc->Free(pidlMain);
				psfFolder->Release();
				pMalloc->Release();
				return;
			}

			// Bind to the folder specified in the new item ID list.
			if(!SUCCEEDED(psfFolder->BindToObject(pidlNextItem, NULL, IID_IShellFolder, (void**)&psfNextFolder))) {
				pMalloc->Free(pidlNextItem);
				pMalloc->Free(pidlMain);
				psfFolder->Release();
				pMalloc->Release();
				return;
			}

			// Release the IShellFolder pointer to the parent folder
			// and set psfFolder equal to the IShellFolder pointer for
			// the current folder.
			psfFolder->Release();
			psfFolder = psfNextFolder;

			// Release the storage for the 1-item item ID list we created
			// just a moment ago and initialize pidlItem so that it points
			// to the next item in pidlMain.
			pMalloc->Free(pidlNextItem);
			pidlItem = GetNextItem (pidlItem);
		}

		// Get a pointer to the item's IContextMenu interface and call
		// IContextMenu::QueryContextMenu to initialize a context menu.
		ppidl = &pidlItem;
		if(SUCCEEDED(psfFolder->GetUIObjectOf(hwnd, 1, (const ITEMIDLIST**)ppidl, (const GUID)IID_IContextMenu, NULL, (void**)&pContextMenu))) {
			hMenu = CreatePopupMenu();
			if(SUCCEEDED(pContextMenu->QueryContextMenu(hMenu, 0, 1, 0x7FFF, CMF_NORMAL))) {
				// Display the context menu.
				nCmd = TrackPopupMenu(hMenu, TPM_LEFTALIGN |
										TPM_LEFTBUTTON | TPM_RIGHTBUTTON | TPM_RETURNCMD,
										point.x, point.y, 0, hwnd, NULL);

				// If a command was selected from the menu, execute it.
				if(nCmd) {
					ici.cbSize = sizeof (CMINVOKECOMMANDINFO);
					ici.fMask = SEE_MASK_FLAG_NO_UI;
					ici.hwnd = hwnd;
					ici.lpVerb = MAKEINTRESOURCE (nCmd - 1);
					ici.lpParameters = NULL;
					ici.lpDirectory = NULL;
					ici.nShow = SW_SHOWNORMAL;
					ici.dwHotKey = 0;
					ici.hIcon = NULL;

					if(SUCCEEDED(pContextMenu->InvokeCommand(&ici))) {
						bResult = TRUE;
					}
				}
			}
			DestroyMenu (hMenu);
			pContextMenu->Release();
		}
	}
	pMalloc->Free (pidlMain);

	// Clean up and return.
	psfFolder->Release();
	pMalloc->Release();

	return;
}
