/*
This is a part of the IconDesk source code.

Copyright (C) 2003 Erik Christiansson, aka Sci
www.alphafish.com
erik@alphafish.com

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/
#include "pidl.h"

LPITEMIDLIST AppendPIDL(LPITEMIDLIST pidlBase, LPITEMIDLIST pidlAdd, LPMALLOC pMalloc)
{
    if(pidlBase == NULL)
        return MakeCopy(pidlAdd);
    if(pidlAdd == NULL)
        return MakeCopy(pidlBase);
    
    LPITEMIDLIST pidlNew;

    UINT cb1 = GetSize(pidlBase) - sizeof(pidlBase->mkid.cb);
    UINT cb2 = GetSize(pidlAdd);

    pidlNew = (LPITEMIDLIST)pMalloc->Alloc(cb1 + cb2);
    if(pidlNew) {
        CopyMemory(pidlNew, pidlBase, cb1);
        CopyMemory(((LPSTR)pidlNew) + cb1, pidlAdd, cb2);
    }
    return pidlNew;
}

/*
void StrRetToBuf(LPSTRRET pstr, LPCITEMIDLIST pidl, LPTSTR pszBuf, UINT cchBuf)
{
	LPMALLOC pMalloc;

	if(NOERROR == SHGetMalloc (&pMalloc)) {
		switch(pstr->uType) {
			case STRRET_WSTR:
			{       
				WideCharToMultiByte(CP_ACP, 0, pstr->pOleStr, -1, pszBuf, cchBuf, NULL, NULL);
				pMalloc->Free(pstr->pOleStr);
				break;
			}
			case STRRET_CSTR:
			{
				lstrcpyn(pszBuf, pstr->cStr, cchBuf);
				break;
			}
			case STRRET_OFFSET:
			{
				lstrcpyn(pszBuf, (LPTSTR)pidl + pstr->uOffset, cchBuf);
				break;
			}
		}
		pMalloc->Release();
	}
}
*/

LPITEMIDLIST GetNextItemID(LPCITEMIDLIST pidl) 
{ 
	// Check for valid pidl.
	if(pidl == NULL)
		return NULL;

	// Get the size of the specified item identifier. 
	int cb = pidl->mkid.cb; 

	// If the size is zero, it is the end of the list. 
	if (cb == 0) 
		return NULL; 

	// Add cb to pidl (casting to increment by bytes). 
	pidl = (LPITEMIDLIST) (((LPBYTE) pidl) + cb); 

	// Return NULL if it is null-terminating, or a pidl otherwise. 
	return (pidl->mkid.cb == 0) ? NULL : (LPITEMIDLIST) pidl; 
} 

UINT GetSize(LPCITEMIDLIST pidl)
{
    UINT cbTotal = 0;
    if (pidl)
    {
        cbTotal += sizeof(pidl->mkid.cb);    // Null terminator
        while (pidl)
        {
            cbTotal += pidl->mkid.cb;
            pidl = GetNextItemID(pidl);
        }
    }
    return cbTotal;
}

LPITEMIDLIST MakeCopy(LPCITEMIDLIST pidl)
{
    LPMALLOC pMalloc;
    HRESULT hr;
    hr = SHGetMalloc(&pMalloc);

    UINT cb = 0;
	LPITEMIDLIST pidlTemp = (LPITEMIDLIST) pidl;
	
// Calculate size of list.
    cb = GetSize(pidl);

    LPITEMIDLIST pidlRet = (LPITEMIDLIST)pMalloc->Alloc(cb);
    if (pidlRet)
        CopyMemory(pidlRet, pidl, cb);

    pMalloc->Release();

    return pidlRet;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//
//  FUNCTION:       GetItemCount
//
//  DESCRIPTION:    Computes the number of item IDs in an item ID list.
//
//  INPUT:          pidl = Pointer to an item ID list.
//
//  RETURNS:        Number of item IDs in the list.
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

UINT GetItemCount (LPITEMIDLIST pidl)
{
	USHORT nLen;
	UINT nCount;
	nCount = 0;
	while ((nLen = pidl->mkid.cb) != 0)
	{
		pidl = GetNextItem (pidl);
		nCount++;
	}
	return nCount;
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//
//  FUNCTION:       GetNextItem
//
//  DESCRIPTION:    Finds the next item in an item ID list.
//
//  INPUT:          pidl = Pointer to an item ID list.
//
//  RETURNS:        Pointer to the next item.
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

LPITEMIDLIST GetNextItem (LPITEMIDLIST pidl)
{
	USHORT nLen;

	if ((nLen = pidl->mkid.cb) == 0)
		return NULL;

	return (LPITEMIDLIST) (((LPBYTE) pidl) + nLen);
}

//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//
//  FUNCTION:       DuplicateItem
//
//  DESCRIPTION:    Makes a copy of the next item in an item ID list.
//
//  INPUT:          pMalloc = Pointer to an IMalloc interface.
//                  pidl    = Pointer to an item ID list.
//
//  RETURNS:        Pointer to an ITEMIDLIST containing the copied item ID.
//
//  NOTES:          It is the caller's responsibility to free the memory
//                  allocated by this function when the item ID is no longer
//                  needed. Example:
//
//                    pidlItem = DuplicateItem (pMalloc, pidl);
//                      .
//                      .
//                      .
//                    pMalloc->Free(pidlItem);
//
//                  Failure to free the ITEMIDLIST will result in memory
//                  leaks.
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

LPITEMIDLIST DuplicateItem (LPMALLOC pMalloc, LPITEMIDLIST pidl)
{
	USHORT nLen;
	LPITEMIDLIST pidlNew;

	nLen = pidl->mkid.cb;
	if (nLen == 0)
		return NULL;

	pidlNew = (LPITEMIDLIST) pMalloc->Alloc (nLen + sizeof (USHORT));
	if (pidlNew == NULL)
		return NULL;

	CopyMemory (pidlNew, pidl, nLen);
	*((USHORT*) (((LPBYTE) pidlNew) + nLen)) = 0;

	return pidlNew;
}
